# parse-git-commit [![npm version][npmv-img]][npmv-url] [![github release][github-release-img]][github-release-url] [![License][license-img]][license-url] 

> An extensible parser for a single commit message that follows Conventional Commits v1 specification

<div id="thetop"></div>

_You might also be interested in [hela][highlighted-link] or in the other [related projects](#related-projects)._

## Quality Assurance :100:

[![bitHound Deps][bithound-code-img]][bithound-code-url] 
[![Code Style Standard][standard-img]][standard-url] 
[![Linux Build][travis-img]][travis-url] 
[![Code Coverage][codecov-img]][codecov-url] 
[![bitHound Score][bithound-score-img]][bithound-score-url] 
[![bitHound Deps][bithound-deps-img]][bithound-deps-url] 
[![Dependencies Status][dependencies-img]][dependencies-url] 

If you have any _how-to_ kind of questions, please read [Code of Conduct](./CODE_OF_CONDUCT.md) and **join the chat** rooms or [open an issue][open-issue-url].  
You may also read the [Contributing Guide](./CONTRIBUTING.md). There, beside _"How to contribute?"_, we describe everything **_stated_** by  the badges.

[![tunnckoCore support][chat-img]][chat-url] 
[![Code Format Prettier][prettier-img]][prettier-url] 
[![Node Security Status][nodesecurity-img]][nodesecurity-url] 
[![Conventional Commits][ccommits-img]][ccommits-url] 
[![Semantic Release][semantic-release-img]][semantic-release-url] 
[![Node Version Required][nodeversion-img]][nodeversion-url] 
[![Renovate App Status][renovate-img]][renovate-url]

<a target="_blank" rel="nofollow" href="https://app.codesponsor.io/link/K7yYzzA5nb2ZDR4GTKmgUdfe/tunnckoCore/parse-git-commit">
  <img alt="Sponsor" width="888" height="68" src="https://app.codesponsor.io/embed/K7yYzzA5nb2ZDR4GTKmgUdfe/tunnckoCore/parse-git-commit.svg" />
</a>
<p></p>

[![All Contributors Spec][all-contributors-img]](#contributors) 
[![Make A Pull Request][prs-welcome-img]][prs-welcome-url] 
[![Newsletter Subscribe][tinyletter-img]][tinyletter-url] 
[![PayPal Author Support][paypal-donate-img]][paypal-donate-url] 
[![Share Love Tweet][share-love-img]][share-love-url] 
[![NPM Downloads Weekly][downloads-weekly-img]][npmv-url] 
[![NPM Downloads Monthly][downloads-monthly-img]][npmv-url] 
[![NPM Downloads Total][downloads-total-img]][npmv-url] 

## Table of Contents
- [Install](#install)
- [API](#api)
- [Related Projects](#related-projects)
- [Contributing](#contributing)
- [Contributors](#contributors)
- [Users](#users)
- [License](#license)

_(TOC generated by [verb](https://github.com/verbose/verb) using [markdown-toc](https://github.com/jonschlinkert/markdown-toc))_

## Install

This project requires [**Node.js**][nodeversion-url] **v8** or above. Install it using [**yarn**](https://yarnpkg.com) **v1** or above / [**npm**](https://www.npmjs.com) **v5** or above.

```
$ yarn add parse-git-commit
```
<!-- 
A browser usage is also possible, thanks to the [unpkg.com](https://unpkg.com) CDN and [Rollup](https://ghub.now.sh/rollup) bundler.  
See available bundles at [`https://unpkg.com/parse-git-commit/dist/browser/`](https://unpkg.com/parse-git-commit/dist/browser/).

> _**Note:** May not work in the browser if some of the [Node.js builtin modules](https://github.com/juliangruber/builtins/blob/master/builtins.json) are used here._
 -->
 
## API
Review carefully the provided examples and the working [tests](./test).

```js
const parseGitCommit = require('parse-git-commit');

const commitMsg1 = `feat(ng-list): Allow custom separator
bla bla bla

BREAKING CHANGE: some breaking change.
Thanks @foobar
`;

const commit = parseGitCommit(commitMsg1);
console.log(commit);
// => { type: 'feat',
//   scope: 'ng-list',
//   subject: 'Allow custom separator',
//   header: 'feat(ng-list): Allow custom separator',
//   body: 'bla bla bla',
//   footer: 'BREAKING CHANGE: some breaking change.\nThanks @stevemao' }

```

It is also extensible if you pass `function` as second argument, so you can add more
properties to the end object.

For example adding `increment` and `mentions` properties like following. Notice that
the `increment` property here is "major" because it detects "BREAKING CHANGE:" somewhere
in the commit.  

That's cool and can be combined with the [semver][]'s `.inc` method like `semver.inc('1.1.0', commit.increment)`

```js
const commit = parseGitCommit(commitMsg1, (commit) => {
  const withIncrement = incrementMapper(commit);
  const withMentions = mentionsMapper(commit);

  return Object.assign({}, commit, withIncrement, withMentions);
});

console.log(commit);
// => { type: 'feat',
//   scope: 'ng-list',
//   subject: 'Allow custom separator',
//   header: 'feat(ng-list): Allow custom separator',
//   body: 'bla bla bla',
//   footer: 'BREAKING CHANGE: some breaking change.\nThanks @stevemao, @foobar',
//   increment: 'major',
//   isBreaking: true,
//   mentions: ['stevemao', 'foobar'] }
```

detects if commit is patch, minor or major type

```js
function incrementMapper (commit) {
  const isBreaking = isBreakingChange(commit);
  let increment = null;

  if (/fix|bugfix|patch/.test(commit.type)) {
    increment = 'patch';
  }
  if (/feat|feature|minor/.test(commit.type)) {
    increment = 'minor';
  }
  if (/break|breaking|major/.test(commit.type) || isBreaking) {
    increment = 'major';
  }

  return { increment, isBreaking };
}

function isBreakingChange ({ subject, body, footer }) {
  const re = 'BREAKING CHANGE:';
  return subject.includes(re) || body.includes(re) || footer.includes(re);
}
```

Collects all mentions from `subject`, `body` and `footer` 
into one single array that will be added to the end "commit" object

```js
const collectMentions = require('collect-mentions');

function mentionsMapper ({ subject, body, footer }) {
  const mentions = []
    .concat(collectMentions(subject))
    .concat(collectMentions(body))
    .concat(collectMentions(footer));

  return { mentions };
}
```

**[back to top](#thetop)**

## Related Projects
Some of these projects are used here or were inspiration for this one, others are just related. So, thanks for your existance! 
- [dush](https://www.npmjs.com/package/dush): Microscopic & functional event emitter in ~350 bytes, extensible through plugins | [homepage](https://github.com/tunnckocore/dush#readme "Microscopic & functional event emitter in ~350 bytes, extensible through plugins")
- [execa](https://www.npmjs.com/package/execa): A better `child_process` | [homepage](https://github.com/sindresorhus/execa#readme "A better `child_process`")
- [hela](https://www.npmjs.com/package/hela): Powerful & flexible task runner framework in 80 lines, based on execa… [more](https://github.com/tunnckoCore/hela#readme) | [homepage](https://github.com/tunnckoCore/hela#readme "Powerful & flexible task runner framework in 80 lines, based on execa. Supports presets, a la ESLint but for tasks & npm scripts")
- [mri](https://www.npmjs.com/package/mri): Quickly scan for CLI flags and arguments | [homepage](https://github.com/lukeed/mri#readme "Quickly scan for CLI flags and arguments")
- [p-map-series](https://www.npmjs.com/package/p-map-series): Map over promises serially | [homepage](https://github.com/sindresorhus/p-map-series#readme "Map over promises serially")

**[back to top](#thetop)**

## Contributing
Please read the [Contributing Guide](./CONTRIBUTING.md) and [Code of Conduct](./CODE_OF_CONDUCT.md) documents for advices.  
For bugs reports and feature requests, [please create an issue][open-issue-url] or join us at our [Flock chat][chat-url] rooms.
  
## Contributors
Thanks to the hard work of [these wonderful people](./CONTRIBUTORS.md) this project is alive and it also follows the [all-contributors](https://github.com/kentcdodds/all-contributors) specification.  
[Pull requests](./CONTRIBUTING.md#opening-a-pull-request), stars and all kind of [contributions](https://opensource.guide/how-to-contribute/#what-it-means-to-contribute) are always welcome.

## Users
You can see who uses `parse-git-commit` in the [USERS.md](./USERS.md) file. Please feel free adding this file if it not exists.  
If you or your organization are using this project, consider adding yourself to the list of users. **Thank You!**

## License
Copyright (c) 2017-present, [Charlike Mike Reagent][author-link] `<olsten.larck@gmail.com>`.  
Released under the [Apache-2.0 License][license-url].

***

_This file was generated by [verb-generate-readme](https://github.com/verbose/verb-generate-readme), v0.6.0, on November 14, 2017._  
_Project scaffolded and managed with [hela][]._

[hela]: https://github.com/tunnckoCore/hela
[semver]: https://github.com/npm/node-semver

<!-- Heading badges -->
[npmv-url]: https://www.npmjs.com/package/parse-git-commit
[npmv-img]: https://img.shields.io/npm/v/parse-git-commit.svg?label=npm%20version

[github-release-url]: https://github.com/tunnckoCore/parse-git-commit/releases/latest
[github-release-img]: https://img.shields.io/github/release/tunnckoCore/parse-git-commit.svg?label=github%20release

[license-url]: https://github.com/tunnckoCore/parse-git-commit/blob/master/LICENSE
[license-img]: https://img.shields.io/npm/l/parse-git-commit.svg
<!-- [license-img]: https://img.shields.io/badge/license-tunnckoCore_1%2E0-blue.svg -->

[downloads-monthly-img]: https://img.shields.io/npm/dm/parse-git-commit.svg

<!-- Front line badges -->
[bithound-score-url]: https://www.bithound.io/github/tunnckoCore/parse-git-commit
[bithound-score-img]: https://www.bithound.io/github/tunnckoCore/parse-git-commit/badges/score.svg

[bithound-code-url]: https://www.bithound.io/github/tunnckoCore/parse-git-commit
[bithound-code-img]: https://www.bithound.io/github/tunnckoCore/parse-git-commit/badges/code.svg

[standard-url]: https://github.com/airbnb/javascript
[standard-img]: https://img.shields.io/badge/code_style-airbnb-brightgreen.svg

[travis-url]: https://travis-ci.org/tunnckoCore/parse-git-commit
[travis-img]: https://img.shields.io/travis/tunnckoCore/parse-git-commit/master.svg?label=linux

[codecov-url]: https://codecov.io/gh/tunnckoCore/parse-git-commit
[codecov-img]: https://img.shields.io/codecov/c/github/tunnckoCore/parse-git-commit/master.svg

[bithound-deps-url]: https://www.bithound.io/github/tunnckoCore/parse-git-commit/dependencies/npm
[bithound-deps-img]: https://www.bithound.io/github/tunnckoCore/parse-git-commit/badges/dependencies.svg

[dependencies-url]: https://david-dm.org/tunnckoCore/parse-git-commit
[dependencies-img]: https://img.shields.io/david/tunnckoCore/parse-git-commit.svg

<!-- Second front of badges -->
[chat-url]: https://tunnckocore.flock.com/?i=cx2xoeofjtj6eo6c
[chat-img]: https://img.shields.io/badge/chat-on_flock-brightgreen.svg

[prettier-url]: https://github.com/prettier/prettier
[prettier-img]: https://img.shields.io/badge/styled_with-prettier-f952a5.svg

[nodesecurity-url]: https://nodesecurity.io/orgs/tunnckocore/projects/todo/master
[nodesecurity-img]: https://nodesecurity.io/orgs/tunnckocore/projects/todo/badge
<!-- the original color of nsp: 
[nodesec-img]: https://img.shields.io/badge/nsp-no_known_vulns-35a9e0.svg -->

[ccommits-url]: https://conventionalcommits.org/
[ccommits-img]: https://img.shields.io/badge/conventional_commits-1.0.0-yellow.svg

[semantic-release-url]: https://github.com/semantic-release/semantic-release
[semantic-release-img]: https://img.shields.io/badge/%20%20%F0%9F%93%A6%F0%9F%9A%80-semantic--release-e10079.svg

[nodeversion-url]: https://nodejs.org/en/download
[nodeversion-img]: https://img.shields.io/node/v/parse-git-commit.svg

[renovate-url]: https://renovateapp.com
[renovate-img]: https://img.shields.io/badge/renovate-enabled-brightgreen.svg

<!-- Third badges line (After CodeSponsor ads) -->
[all-contributors-img]: https://img.shields.io/github/contributors/tunnckoCore/parse-git-commit.svg?label=all%20contributors&colorB=ffa500

[prs-welcome-img]: https://img.shields.io/badge/PRs-welcome-brightgreen.svg
[prs-welcome-url]: http://makeapullrequest.com

[tinyletter-url]: https://tinyletter.com/tunnckoCore
[tinyletter-img]: https://img.shields.io/badge/join-newsletter-9caaf8.svg

[paypal-donate-url]: https://paypal.me/tunnckoCore/10
[paypal-donate-img]: https://img.shields.io/badge/$-support-f47721.svg

[downloads-weekly-img]: https://img.shields.io/npm/dw/parse-git-commit.svg
[downloads-total-img]: https://img.shields.io/npm/dt/parse-git-commit.svg

<!-- Miscellaneous -->
[share-love-url]: https://twitter.com/intent/tweet?text=https://github.com/tunnckoCore/parse-git-commit&via=tunnckoCore
[share-love-img]: https://img.shields.io/badge/share-love-1da1f2.svg
[open-issue-url]: https://github.com/tunnckoCore/parse-git-commit/issues/new

[highlighted-link]: https://ghub.now.sh/hela
[author-link]: https://charlike.online

